/* w7ttu.cpp --------------------------------------------------------------------------//
 * 
 * (C) Copyright Ideum Inc 2012
 * 
 * This file is part of Windows 7 to TUIO Touch Utility
 *
 * Windows 7 to TUIO Touch Utility is free software: you can redistribute it
 * it under the terms of the GNU Affero General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Windows 7 to TUIO Touch Utility is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Windows 7 to TUIO Touch Utility.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * Author: John Butler
 * ------------------------------------------------------------------------------------*/


#include "stdafx.h"

#include <Windows.h>
#include <WinUser.h>
#include <vector>
#include <string.h>
#include <sstream>
#include <ws2tcpip.h>
#include <boost/format.hpp>
#include <stdio.h>

#include "opensoundcontrol.hpp"
#include "resource.h"
#include "w7ttu.hpp"

#pragma comment(lib, "ws2_32.lib")

const bool DEBUG = true;

// Process all Windows messages
LRESULT CALLBACK WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{

	switch(msg){
	case WM_TIMER:
		switch(wParam){
		// time to send an update message
		case MESSAGE_TIMER:
			send_update();
			if(DEBUG){
				printf(("-" + points_to_string() + "\n").c_str());
				//printf((points_to_tuio() + "\n").c_str());
			}
			break;
		}
		break;
	case WM_KEYDOWN:
		// Quit the application when the escape key is pressed
		switch(wParam){
		case VK_ESCAPE:
			WSACleanup();
			DestroyWindow(hWnd);
			exit(1);
		}
	
		break;
	case WM_TOUCH:
		processTouch(hWnd, wParam, lParam);
		break;
	case WM_CLOSE:
		DestroyWindow(hWnd);
		break;
	case WM_DESTROY:
		PostQuitMessage(0);
		break;

	default:
		return DefWindowProc(hWnd, msg, wParam, lParam);
	}
	return 0;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	if(DEBUG){
		AllocConsole();
		freopen("CONOUT$", "wb", stdout);
		freopen("CONOUT$", "wb", stderr);
	}

	WNDCLASSEX wcx;
	HWND hWnd;
	MSG msg;
	int iResult;
	WSADATA udpwsaData;
	UDPSock = INVALID_SOCKET;

	id = 0;
	frame = 0;


	// Get the screen resolution in pixels
	cxscreen = GetSystemMetrics(SM_CXSCREEN);
	cyscreen = GetSystemMetrics(SM_CYSCREEN);

	// Init Winsock
	iResult = WSAStartup(MAKEWORD(2,2), &udpwsaData);
	if(iResult != 0){
		MessageBox(NULL, "WSAStartup failed.\n", "Error!", MB_OK);
		WSACleanup();
		return 1;
	}

	const char* hname = "127.0.0.1";
	udpAddr.sin_family = AF_INET;
	udpAddr.sin_port = htons((short)DEFAULT_PORT);
	if ((udpAddr.sin_addr.s_addr = inet_addr(DEFAULT_HOST)) == INADDR_NONE){
		struct hostent *host=NULL;
		host = gethostbyname(DEFAULT_HOST);
		if(host){
			CopyMemory(&udpAddr.sin_addr, host->h_addr_list[0], host->h_length);
		}
		else{
			MessageBox(NULL, "gethostbyname failed", "Error!", 0);
			WSACleanup();
			return 1;
		}
	}
	
	// create socket
	UDPSock = socket(AF_INET, SOCK_DGRAM, 0);
	if(UDPSock == INVALID_SOCKET){
		MessageBox(NULL, "socket failed.", "Error!", MB_OK);
		freeaddrinfo(result);
		WSACleanup();
		return 1;
	}
	
	wcx.cbSize = sizeof(WNDCLASSEX);
	wcx.style = 0;
	wcx.lpfnWndProc = WndProc;
	wcx.cbClsExtra = 0;
	wcx.cbWndExtra = 0;
	wcx.hInstance = hInstance;
	wcx.hIcon = LoadIcon(hInstance, LPCSTR(IDI_W7TTU));
	wcx.hCursor = LoadCursor(NULL, IDC_ARROW);
	wcx.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
	wcx.lpszMenuName = NULL;
	wcx.lpszClassName = g_szClassName;
	wcx.hIconSm = LoadIcon(NULL, IDI_APPLICATION);
	

	if(!RegisterClassEx(&wcx)){
		MessageBox(NULL, "Window registration failed!", "Error!", MB_ICONEXCLAMATION | MB_OK);
		return 0;
	}

	// create window handle
	hWnd = CreateWindowEx(WS_EX_CLIENTEDGE | WS_EX_LAYERED, g_szClassName, "Win7 to TUIO Touch Utility", WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, CW_USEDEFAULT, 240, 120, NULL, NULL, hInstance, NULL);
	
	// set window transparency level 
	SetLayeredWindowAttributes(hWnd, RGB(0,0,0), 1, LWA_ALPHA);

	if(hWnd == NULL){
		MessageBox(NULL, "Window creation failed!", "Error!", MB_ICONEXCLAMATION | MB_OK);
		return 0;
	}

	

	// remove window menu controls and border
	DWORD style = ::GetWindowLongA(hWnd, GWL_STYLE);
	DWORD junk = WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX | WS_MAXIMIZEBOX | WS_THICKFRAME;
	DWORD newstyle = style & ~junk;
	::SetWindowLongA(hWnd, GWL_STYLE, newstyle | WS_EX_LAYERED);
	::SetWindowPos(hWnd, NULL, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_NOZORDER | SWP_NOACTIVATE | SWP_FRAMECHANGED);
	SetMenu(hWnd, NULL);
	

	// move window to fullscreen
	HDC hdc = ::GetWindowDC(hWnd);
	::SetWindowPos(hWnd, NULL, 0, 0, ::GetDeviceCaps(hdc, HORZRES), ::GetDeviceCaps(hdc, VERTRES), SWP_FRAMECHANGED);

	RegisterTouchWindow(hWnd, 0);

	//ShowWindow(hWnd, nCmdShow); // let the user chose how the application starts
	ShowWindow(hWnd, SW_SHOWMINNOACTIVE); // always start the application minimized (to help when moving windows around)

	UpdateWindow(hWnd);

	// Set a timer for sending TUIO updates
	SetTimer(hWnd, MESSAGE_TIMER, MSG_PER, NULL);

	while(GetMessage(&msg, NULL, 0, 0) > 0){
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	return msg.wParam;
}